"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.app = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const body_parser_1 = __importDefault(require("body-parser"));
const cookie_parser_1 = __importDefault(require("cookie-parser"));
const express_1 = __importDefault(require("express"));
const helmet_1 = __importDefault(require("helmet"));
const kv_logger_1 = require("kv-logger");
const app_error_1 = require("./core/app-error");
const config_1 = require("./core/config");
const i18n_1 = require("./core/i18n");
const middleware_1 = require("./core/middleware");
const accessKeys_1 = require("./routes/accessKeys");
const account_1 = require("./routes/account");
const apps_1 = require("./routes/apps");
const auth_1 = require("./routes/auth");
const index_1 = require("./routes/index");
const indexV1_1 = require("./routes/indexV1");
const users_1 = require("./routes/users");
exports.app = (0, express_1.default)();
exports.app.use((0, helmet_1.default)({
    contentSecurityPolicy: false,
}));
// view engine setup
exports.app.set('views', path_1.default.join(__dirname, '../views'));
exports.app.set('view engine', 'pug');
// translations
exports.app.use(i18n_1.i18n.init);
exports.app.use(body_parser_1.default.json());
exports.app.use(body_parser_1.default.urlencoded({ extended: false }));
exports.app.use((0, cookie_parser_1.default)());
exports.app.use(express_1.default.static(path_1.default.join(__dirname, '../public')));
exports.app.use(middleware_1.withLogger);
exports.app.all('*', (req, res, next) => {
    res.header('Access-Control-Allow-Origin', '*');
    res.header('Access-Control-Allow-Headers', 'Origin, X-Requested-With, Content-Type, Accept, Authorization, X-CodePush-Plugin-Version, X-CodePush-Plugin-Name, X-CodePush-SDK-Version, X-Request-Id');
    res.header('Access-Control-Allow-Methods', 'PUT,POST,GET,PATCH,DELETE,OPTIONS');
    next();
});
kv_logger_1.logger.debug(`config common.storageType value: ${config_1.config.common.storageType}`);
// config local storage
if (config_1.config.common.storageType === 'local') {
    const localStorageDir = config_1.config.local.storageDir;
    if (localStorageDir) {
        kv_logger_1.logger.debug(`config common.storageDir value: ${localStorageDir}`);
        if (!fs_1.default.existsSync(localStorageDir)) {
            const e = new Error(`Please create dir ${localStorageDir}`);
            kv_logger_1.logger.error(e);
            throw e;
        }
        try {
            kv_logger_1.logger.debug('checking storageDir fs.W_OK | fs.R_OK');
            // eslint-disable-next-line no-bitwise
            fs_1.default.accessSync(localStorageDir, fs_1.default.constants.W_OK | fs_1.default.constants.R_OK);
            kv_logger_1.logger.debug('storageDir fs.W_OK | fs.R_OK is ok');
        }
        catch (e) {
            kv_logger_1.logger.error(e);
            throw e;
        }
        kv_logger_1.logger.debug(`static download uri value: ${config_1.config.local.public}`);
        exports.app.use(config_1.config.local.public, express_1.default.static(localStorageDir));
    }
    else {
        kv_logger_1.logger.error('please config local storageDir');
    }
}
// config routes
// code-push-client routes
exports.app.use('/', index_1.indexRouter);
exports.app.use('/v0.1/public/codepush', indexV1_1.indexV1Router);
// code-push-cli routes
exports.app.use('/accessKeys', accessKeys_1.accessKeysRouter);
exports.app.use('/apps', apps_1.appsRouter);
exports.app.use('/account', account_1.accountRouter);
// code-push-server routes
exports.app.use('/auth', auth_1.authRouter);
exports.app.use('/users', users_1.usersRouter);
// 404 handler
// eslint-disable-next-line @typescript-eslint/no-unused-vars
exports.app.use((req, res, next) => {
    throw new app_error_1.NotFound(`${req.method} ${req.url} not found`);
});
// error handler
// eslint-disable-next-line @typescript-eslint/no-unused-vars
exports.app.use((err, req, res, next) => {
    const thisLogger = req.logger || kv_logger_1.logger;
    if (err instanceof app_error_1.AppError) {
        res.status(err.status).send(err.message);
        thisLogger.debug(err);
    }
    else {
        res.status(500).send(err.message);
        thisLogger.error(err);
    }
});
