"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.accountManager = void 0;
const lodash_1 = __importDefault(require("lodash"));
const moment_1 = __importDefault(require("moment"));
const validator_1 = __importDefault(require("validator"));
const collaborators_1 = require("../../models/collaborators");
const user_tokens_1 = require("../../models/user_tokens");
const users_1 = require("../../models/users");
const app_error_1 = require("../app-error");
const config_1 = require("../config");
const connections_1 = require("../utils/connections");
const security_1 = require("../utils/security");
const email_manager_1 = require("./email-manager");
const LOGIN_LIMIT_PRE = 'LOGIN_LIMIT_PRE_';
const REGISTER_CODE = 'REGISTER_CODE_';
const EXPIRED = 1200;
const EXPIRED_SPEED = 10;
class AccountManager {
    collaboratorCan(uid, appName, logger) {
        return this.getCollaborator(uid, appName).then((data) => {
            if (!data) {
                logger.debug(`collaboratorCan App ${appName} not exists.`);
                throw new app_error_1.AppError(`App ${appName} not exists.`);
            }
            return data;
        });
    }
    ownerCan(uid, appName, logger) {
        return this.getCollaborator(uid, appName).then((data) => {
            if (!data) {
                logger.debug(`ownerCan App ${appName} not exists.`);
                throw new app_error_1.AppError(`App ${appName} not exists.`);
            }
            if (!lodash_1.default.eq(lodash_1.default.get(data, 'roles'), 'Owner')) {
                logger.debug(`ownerCan Permission Deny, You are not owner!`);
                throw new app_error_1.AppError('Permission Deny, You are not owner!');
            }
            return data;
        });
    }
    findUserByEmail(email) {
        return users_1.Users.findOne({ where: { email } }).then((data) => {
            if (lodash_1.default.isEmpty(data)) {
                throw new app_error_1.AppError(`${email} does not exist.`);
            }
            else {
                return data;
            }
        });
    }
    getAllAccessKeyByUid(uid) {
        return user_tokens_1.UserTokens.findAll({
            where: { uid },
            order: [['id', 'DESC']],
        }).then((tokens) => {
            return lodash_1.default.map(tokens, (v) => {
                return {
                    name: '(hidden)',
                    createdTime: (0, moment_1.default)(v.created_at).valueOf(),
                    createdBy: v.created_by,
                    expires: (0, moment_1.default)(v.expires_at).valueOf(),
                    friendlyName: v.name,
                    description: v.description,
                };
            });
        });
    }
    isExsitAccessKeyName(uid, friendlyName) {
        return user_tokens_1.UserTokens.findOne({
            where: { uid, name: friendlyName },
        });
    }
    createAccessKey(uid, newAccessKey, ttl, friendlyName, createdBy, description) {
        return user_tokens_1.UserTokens.create({
            uid,
            name: friendlyName,
            tokens: newAccessKey,
            description,
            created_by: createdBy,
            expires_at: (0, moment_1.default)()
                .add(ttl / 1000, 'seconds')
                .format('YYYY-MM-DD HH:mm:ss'),
            created_at: (0, moment_1.default)().format('YYYY-MM-DD HH:mm:ss'),
        });
    }
    login(account, password) {
        if (lodash_1.default.isEmpty(account)) {
            return Promise.reject(new app_error_1.AppError('请您输入邮箱地址'));
        }
        if (lodash_1.default.isEmpty(password)) {
            return Promise.reject(new app_error_1.AppError('请您输入密码'));
        }
        let where = {};
        if (validator_1.default.isEmail(account)) {
            where = { email: account };
        }
        else {
            where = { username: account };
        }
        const { tryLoginTimes } = config_1.config.common;
        return users_1.Users.findOne({ where })
            .then((users) => {
            if (lodash_1.default.isEmpty(users)) {
                throw new app_error_1.AppError('您输入的邮箱或密码有误');
            }
            return users;
        })
            .then((users) => {
            if (tryLoginTimes > 0) {
                const loginKey = `${LOGIN_LIMIT_PRE}${users.id}`;
                return connections_1.redisClient.get(loginKey).then((loginErrorTimes) => {
                    if (Number(loginErrorTimes) > tryLoginTimes) {
                        throw new app_error_1.AppError(`您输入密码错误次数超过限制，帐户已经锁定`);
                    }
                    return users;
                });
            }
            return users;
        })
            .then((users) => {
            if (!(0, security_1.passwordVerifySync)(password, users.password)) {
                if (tryLoginTimes > 0) {
                    const loginKey = `${LOGIN_LIMIT_PRE}${users.id}`;
                    connections_1.redisClient.exists(loginKey).then((isExists) => {
                        if (!isExists) {
                            const expires = (0, moment_1.default)().endOf('day').unix() - (0, moment_1.default)().unix();
                            connections_1.redisClient.setEx(loginKey, expires, '1');
                            return;
                        }
                        connections_1.redisClient.incr(loginKey);
                    });
                }
                throw new app_error_1.AppError('您输入的邮箱或密码有误');
            }
            else {
                return users;
            }
        });
    }
    sendRegisterCode(email) {
        if (lodash_1.default.isEmpty(email)) {
            return Promise.reject(new app_error_1.AppError('请您输入邮箱地址'));
        }
        return users_1.Users.findOne({ where: { email } })
            .then((u) => {
            if (u) {
                throw new app_error_1.AppError(`"${email}" 已经注册过，请更换邮箱注册`);
            }
        })
            .then(() => {
            // 将token临时存储到redis
            const token = (0, security_1.randToken)(40);
            return connections_1.redisClient
                .setEx(`${REGISTER_CODE}${(0, security_1.md5)(email)}`, EXPIRED, token)
                .then(() => {
                return token;
            });
        })
            .then((token) => {
            // 将token发送到用户邮箱
            return email_manager_1.emailManager.sendRegisterCodeMail(email, token);
        });
    }
    checkRegisterCode(email, token) {
        return users_1.Users.findOne({ where: { email } })
            .then((u) => {
            if (u) {
                throw new app_error_1.AppError(`"${email}" 已经注册过，请更换邮箱注册`);
            }
        })
            .then(() => {
            const registerKey = `${REGISTER_CODE}${(0, security_1.md5)(email)}`;
            return connections_1.redisClient.get(registerKey).then((storageToken) => {
                if (lodash_1.default.isEmpty(storageToken)) {
                    throw new app_error_1.AppError(`验证码已经失效，请您重新获取`);
                }
                if (!lodash_1.default.eq(token, storageToken)) {
                    connections_1.redisClient.ttl(registerKey).then((ttl) => {
                        if (ttl > 0) {
                            connections_1.redisClient.expire(registerKey, ttl - EXPIRED_SPEED);
                        }
                    });
                    throw new app_error_1.AppError(`您输入的验证码不正确，请重新输入`);
                }
                return storageToken;
            });
        });
    }
    register(email, password) {
        return users_1.Users.findOne({ where: { email } })
            .then((u) => {
            if (u) {
                throw new app_error_1.AppError(`"${email}" 已经注册过，请更换邮箱注册`);
            }
        })
            .then(() => {
            const identical = (0, security_1.randToken)(9);
            return users_1.Users.create({
                email,
                password: (0, security_1.passwordHashSync)(password),
                identical,
            });
        });
    }
    changePassword(uid, oldPassword, newPassword) {
        if (!lodash_1.default.isString(newPassword) || newPassword.length < 6) {
            return Promise.reject(new app_error_1.AppError('请您输入6～20位长度的新密码'));
        }
        return users_1.Users.findOne({ where: { id: uid } })
            .then((u) => {
            if (!u) {
                throw new app_error_1.AppError(`未找到用户信息`);
            }
            return u;
        })
            .then((u) => {
            const isEq = (0, security_1.passwordVerifySync)(oldPassword, u.get('password'));
            if (!isEq) {
                throw new app_error_1.AppError(`您输入的旧密码不正确，请重新输入`);
            }
            u.set('password', (0, security_1.passwordHashSync)(newPassword));
            u.set('ack_code', (0, security_1.randToken)(5));
            return u.save();
        });
    }
    getCollaborator(uid, appName) {
        return (0, collaborators_1.findCollaboratorsByAppNameAndUid)(uid, appName);
    }
}
exports.accountManager = new AccountManager();
