"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dataCenterManager = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const lodash_1 = __importDefault(require("lodash"));
const app_error_1 = require("../app-error");
const config_1 = require("../config");
const common_1 = require("../utils/common");
const security_1 = require("../utils/security");
const MANIFEST_FILENAME = 'manifest.json';
const CONTENTS_NAME = 'contents';
class DataCenterManager {
    getDataDir() {
        return config_1.config.common.dataDir;
    }
    hasPackageStoreSync(packageHash) {
        const dataDir = this.getDataDir();
        const packageHashPath = path_1.default.join(dataDir, packageHash);
        const manifestFile = path_1.default.join(packageHashPath, MANIFEST_FILENAME);
        const contentPath = path_1.default.join(packageHashPath, CONTENTS_NAME);
        return fs_1.default.existsSync(manifestFile) && fs_1.default.existsSync(contentPath);
    }
    getPackageInfo(packageHash) {
        if (this.hasPackageStoreSync(packageHash)) {
            const dataDir = this.getDataDir();
            const packageHashPath = path_1.default.join(dataDir, packageHash);
            const manifestFile = path_1.default.join(packageHashPath, MANIFEST_FILENAME);
            const contentPath = path_1.default.join(packageHashPath, CONTENTS_NAME);
            return this.buildPackageInfo(packageHash, packageHashPath, contentPath, manifestFile);
        }
        throw new app_error_1.AppError("can't get PackageInfo");
    }
    buildPackageInfo(packageHash, packageHashPath, contentPath, manifestFile) {
        return {
            packageHash,
            path: packageHashPath,
            contentPath,
            manifestFilePath: manifestFile,
        };
    }
    validateStore(providePackageHash, logger) {
        const dataDir = this.getDataDir();
        const packageHashPath = path_1.default.join(dataDir, providePackageHash);
        const manifestFile = path_1.default.join(packageHashPath, MANIFEST_FILENAME);
        const contentPath = path_1.default.join(packageHashPath, CONTENTS_NAME);
        if (!this.hasPackageStoreSync(providePackageHash)) {
            logger.debug(`validateStore providePackageHash not exist`);
            return Promise.resolve(false);
        }
        return (0, security_1.calcAllFileSha256)(contentPath).then((manifestJson) => {
            const packageHash = (0, security_1.packageHashSync)(manifestJson);
            let manifestJsonLocal;
            try {
                manifestJsonLocal = JSON.parse(fs_1.default.readFileSync(manifestFile, { encoding: 'utf8' }));
            }
            catch (e) {
                logger.debug(`validateStore manifestFile contents invilad`);
                return false;
            }
            const packageHashLocal = (0, security_1.packageHashSync)(manifestJsonLocal);
            if (lodash_1.default.eq(providePackageHash, packageHash) &&
                lodash_1.default.eq(providePackageHash, packageHashLocal)) {
                logger.debug(`validateStore store files is ok`);
                return true;
            }
            logger.debug(`validateStore store files broken`);
            return false;
        });
    }
    storePackage(sourceDst, force, logger) {
        return (0, security_1.calcAllFileSha256)(sourceDst).then((manifestJson) => {
            const packageHash = (0, security_1.packageHashSync)(manifestJson);
            const dataDir = this.getDataDir();
            const packageHashPath = path_1.default.join(dataDir, packageHash);
            const manifestFile = path_1.default.join(packageHashPath, MANIFEST_FILENAME);
            const contentPath = path_1.default.join(packageHashPath, CONTENTS_NAME);
            return this.validateStore(packageHash, logger).then((isValidate) => {
                if (!force && isValidate) {
                    return this.buildPackageInfo(packageHash, packageHashPath, contentPath, manifestFile);
                }
                return (0, common_1.createEmptyFolder)(packageHashPath).then(() => {
                    return (0, common_1.copy)(sourceDst, contentPath).then(() => {
                        const manifestString = JSON.stringify(manifestJson);
                        fs_1.default.writeFileSync(manifestFile, manifestString);
                        return this.buildPackageInfo(packageHash, packageHashPath, contentPath, manifestFile);
                    });
                });
            });
        });
    }
}
exports.dataCenterManager = new DataCenterManager();
