"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.packageManager = void 0;
const fs_1 = __importDefault(require("fs"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const formidable_1 = __importDefault(require("formidable"));
const lodash_1 = __importDefault(require("lodash"));
const sequelize_1 = require("sequelize");
const slash_1 = __importDefault(require("slash"));
const yazl_1 = __importDefault(require("yazl"));
const apps_1 = require("../../models/apps");
const deployments_1 = require("../../models/deployments");
const deployments_history_1 = require("../../models/deployments_history");
const deployments_versions_1 = require("../../models/deployments_versions");
const packages_1 = require("../../models/packages");
const packages_diff_1 = require("../../models/packages_diff");
const packages_metrics_1 = require("../../models/packages_metrics");
const app_error_1 = require("../app-error");
const const_1 = require("../const");
const common_1 = require("../utils/common");
const connections_1 = require("../utils/connections");
const qetag_1 = require("../utils/qetag");
const security_1 = require("../utils/security");
const storage_1 = require("../utils/storage");
const datacenter_manager_1 = require("./datacenter-manager");
class PackageManager {
    getMetricsbyPackageId(packageId) {
        return packages_metrics_1.PackagesMetrics.findOne({ where: { package_id: packageId } });
    }
    findPackageInfoByDeploymentIdAndLabel(deploymentId, label) {
        return packages_1.Packages.findOne({ where: { deployment_id: deploymentId, label } });
    }
    findLatestPackageInfoByDeployVersion(deploymentsVersionsId, logger) {
        return deployments_versions_1.DeploymentsVersions.findByPk(deploymentsVersionsId).then((deploymentsVersions) => {
            if (!deploymentsVersions || deploymentsVersions.current_package_id < 0) {
                const e = new app_error_1.AppError('not found last packages');
                logger.debug(e);
                throw e;
            }
            return packages_1.Packages.findByPk(deploymentsVersions.current_package_id);
        });
    }
    parseReqFile(req, logger) {
        logger.debug('parseReqFile');
        return new Promise((resolve, reject) => {
            const form = (0, formidable_1.default)();
            form.parse(req, (err, fields, files) => {
                if (err) {
                    reject(new app_error_1.AppError('upload error'));
                    return;
                }
                if (lodash_1.default.isEmpty(fields.packageInfo) || lodash_1.default.isEmpty(lodash_1.default.get(files, 'package'))) {
                    logger.debug('parseReqFile upload info lack');
                    reject(new app_error_1.AppError('upload info lack'));
                    return;
                }
                logger.debug('parseReqFile is ok');
                resolve({
                    packageInfo: JSON.parse(fields.packageInfo),
                    package: files.package,
                });
            });
        });
    }
    createDeploymentsVersionIfNotExist(deploymentId, appVersion, minVersion, maxVersion, t, logger) {
        return deployments_versions_1.DeploymentsVersions.findOrCreate({
            where: {
                deployment_id: deploymentId,
                app_version: appVersion,
                min_version: minVersion,
                max_version: maxVersion,
            },
            defaults: { current_package_id: 0 },
            transaction: t,
        }).then(([data, created]) => {
            if (created) {
                logger.debug(`createDeploymentsVersionIfNotExist findOrCreate version ${appVersion}`);
            }
            logger.debug(`createDeploymentsVersionIfNotExist version data:`, data.get());
            return data;
        });
    }
    isMatchPackageHash(packageId, packageHash, logger) {
        if (lodash_1.default.lt(packageId, 0)) {
            logger.debug(`isMatchPackageHash packageId is 0`);
            return Promise.resolve(false);
        }
        return packages_1.Packages.findByPk(packageId).then((data) => {
            if (data && lodash_1.default.eq(data.get('package_hash'), packageHash)) {
                logger.debug(`isMatchPackageHash data:`, data.get());
                logger.debug(`isMatchPackageHash packageHash exist`);
                return true;
            }
            logger.debug(`isMatchPackageHash package is null`);
            return false;
        });
    }
    createPackage(deploymentId, appVersion, packageHash, manifestHash, blobHash, params, logger) {
        const releaseMethod = params.releaseMethod || const_1.RELEASE_METHOD_UPLOAD;
        const releaseUid = params.releaseUid || 0;
        const isMandatory = params.isMandatory || 0;
        const size = params.size || 0;
        const rollout = params.rollout || 100;
        const description = params.description || '';
        const originalLabel = params.originalLabel || '';
        const isDisabled = params.isDisabled || 0;
        const originalDeployment = params.originalDeployment || '';
        return (0, deployments_1.generateDeploymentsLabelId)(deploymentId).then((labelId) => {
            return connections_1.sequelize.transaction((t) => {
                return this.createDeploymentsVersionIfNotExist(deploymentId, appVersion, params.min_version, params.max_version, t, logger).then((deploymentsVersions) => {
                    return packages_1.Packages.create({
                        deployment_version_id: deploymentsVersions.id,
                        deployment_id: deploymentId,
                        description,
                        package_hash: packageHash,
                        blob_url: blobHash,
                        size,
                        manifest_blob_url: manifestHash,
                        release_method: releaseMethod,
                        label: `v${labelId}`,
                        released_by: releaseUid,
                        is_mandatory: isMandatory,
                        is_disabled: isDisabled,
                        rollout,
                        original_label: originalLabel,
                        original_deployment: originalDeployment,
                    }, { transaction: t }).then((packages) => {
                        deploymentsVersions.set('current_package_id', packages.id);
                        return Promise.all([
                            deploymentsVersions.save({ transaction: t }),
                            deployments_1.Deployments.update({ last_deployment_version_id: deploymentsVersions.id }, { where: { id: deploymentId }, transaction: t }),
                            packages_metrics_1.PackagesMetrics.create({ package_id: packages.id }, { transaction: t }),
                            deployments_history_1.DeploymentsHistory.create({ deployment_id: deploymentId, package_id: packages.id }, { transaction: t }),
                        ]).then(() => packages);
                    });
                });
            });
        });
    }
    downloadPackageAndExtract(workDirectoryPath, packageHash, blobHash, logger) {
        return datacenter_manager_1.dataCenterManager.validateStore(packageHash, logger).then((isValidate) => {
            if (isValidate) {
                return datacenter_manager_1.dataCenterManager.getPackageInfo(packageHash);
            }
            const downloadURL = (0, common_1.getBlobDownloadUrl)(blobHash);
            return (0, common_1.createFileFromRequest)(downloadURL, path_1.default.join(workDirectoryPath, blobHash), logger).then(() => {
                return (0, common_1.unzipFile)(path_1.default.join(workDirectoryPath, blobHash), path_1.default.join(workDirectoryPath, 'current'), logger).then((outputPath) => {
                    return datacenter_manager_1.dataCenterManager.storePackage(outputPath, true, logger);
                });
            });
        });
    }
    zipDiffPackage(fileName, files, baseDirectoryPath, hotCodePushFile) {
        return new Promise((resolve, reject) => {
            const zipFile = new yazl_1.default.ZipFile();
            const writeStream = fs_1.default.createWriteStream(fileName);
            writeStream.on('error', (error) => {
                reject(error);
            });
            zipFile.on('error', (error) => {
                reject(error);
            });
            zipFile.outputStream
                .pipe(writeStream)
                .on('error', (error) => {
                reject(error);
            })
                .on('close', () => {
                resolve({ isTemporary: true, path: fileName });
            });
            for (let i = 0; i < files.length; i += 1) {
                const file = files[i];
                zipFile.addFile(path_1.default.join(baseDirectoryPath, file), (0, slash_1.default)(file));
            }
            zipFile.addFile(hotCodePushFile, const_1.DIFF_MANIFEST_FILE_NAME);
            zipFile.end();
        });
    }
    generateOneDiffPackage(workDirectoryPath, packageId, originDataCenter, oldPackageDataCenter, diffPackageHash, diffManifestBlobHash, logger) {
        return packages_diff_1.PackagesDiff.findOne({
            where: {
                package_id: packageId,
                diff_against_package_hash: diffPackageHash,
            },
        }).then((diffPackage) => {
            if (!lodash_1.default.isEmpty(diffPackage)) {
                return undefined;
            }
            logger.debug('generateOneDiffPackage', {
                packageId,
                originDataCenter,
                oldPackageDataCenter,
            });
            const downloadURL = (0, common_1.getBlobDownloadUrl)(diffManifestBlobHash);
            return (0, common_1.createFileFromRequest)(downloadURL, path_1.default.join(workDirectoryPath, diffManifestBlobHash), logger).then(() => {
                const dataCenterContentPath = path_1.default.join(workDirectoryPath, 'dataCenter');
                (0, common_1.copySync)(originDataCenter.contentPath, dataCenterContentPath);
                // const oldPackageDataCenterContentPath = oldPackageDataCenter.contentPath;
                const originManifestJson = JSON.parse(fs_1.default.readFileSync(originDataCenter.manifestFilePath, 'utf8'));
                const diffManifestJson = JSON.parse(fs_1.default.readFileSync(path_1.default.join(workDirectoryPath, diffManifestBlobHash), 'utf8'));
                const json = (0, common_1.diffCollectionsSync)(originManifestJson, diffManifestJson);
                const files = lodash_1.default.concat(json.diff, json.collection1Only);
                const hotcodepush = { deletedFiles: json.collection2Only, patchedFiles: [] };
                const hotCodePushFile = path_1.default.join(workDirectoryPath, `${diffManifestBlobHash}_hotcodepush`);
                fs_1.default.writeFileSync(hotCodePushFile, JSON.stringify(hotcodepush));
                const fileName = path_1.default.join(workDirectoryPath, `${diffManifestBlobHash}.zip`);
                return this.zipDiffPackage(fileName, files, dataCenterContentPath, hotCodePushFile).then((data) => {
                    return (0, qetag_1.qetag)(data.path, logger).then((diffHash) => {
                        return (0, storage_1.uploadFileToStorage)(diffHash, fileName, logger).then(() => {
                            const stats = fs_1.default.statSync(fileName);
                            return packages_diff_1.PackagesDiff.create({
                                package_id: packageId,
                                diff_against_package_hash: diffPackageHash,
                                diff_blob_url: diffHash,
                                diff_size: stats.size,
                            });
                        });
                    });
                });
            });
        });
    }
    createDiffPackagesByLastNums(appId, originalPackage, num, logger) {
        const packageId = originalPackage.id;
        return Promise.all([
            packages_1.Packages.findAll({
                where: {
                    deployment_version_id: originalPackage.deployment_version_id,
                    id: { [sequelize_1.Op.lt]: packageId },
                },
                order: [['id', 'desc']],
                limit: num,
            }),
            packages_1.Packages.findAll({
                where: {
                    deployment_version_id: originalPackage.deployment_version_id,
                    id: { [sequelize_1.Op.lt]: packageId },
                },
                order: [['id', 'asc']],
                limit: 2,
            }),
        ])
            .then(([lastNumsPackages, basePackages]) => {
            return lodash_1.default.uniqBy(lodash_1.default.unionBy(lastNumsPackages, basePackages, 'id'), 'package_hash');
        })
            .then((lastNumsPackages) => {
            return this.createDiffPackages(originalPackage, lastNumsPackages, logger);
        });
    }
    createDiffPackages(originalPackage, destPackages, logger) {
        if (!lodash_1.default.isArray(destPackages)) {
            return Promise.reject(new app_error_1.AppError('第二个参数必须是数组'));
        }
        if (destPackages.length <= 0) {
            return null;
        }
        const packageHash = lodash_1.default.get(originalPackage, 'package_hash');
        // const manifest_blob_url = _.get(originalPackage, 'manifest_blob_url');
        const blobUrl = lodash_1.default.get(originalPackage, 'blob_url');
        const workDirectoryPath = path_1.default.join(os_1.default.tmpdir(), `codepush_${(0, security_1.randToken)(32)}`);
        logger.debug('createDiffPackages using dir', { workDirectoryPath });
        return (0, common_1.createEmptyFolder)(workDirectoryPath)
            .then(() => this.downloadPackageAndExtract(workDirectoryPath, packageHash, blobUrl, logger))
            .then((originDataCenter) => Promise.all(destPackages.map((v) => {
            const diffWorkDirectoryPath = path_1.default.join(workDirectoryPath, lodash_1.default.get(v, 'package_hash'));
            (0, common_1.createEmptyFolderSync)(diffWorkDirectoryPath);
            return this.downloadPackageAndExtract(diffWorkDirectoryPath, lodash_1.default.get(v, 'package_hash'), lodash_1.default.get(v, 'blob_url'), logger).then((oldPackageDataCenter) => this.generateOneDiffPackage(diffWorkDirectoryPath, originalPackage.id, originDataCenter, oldPackageDataCenter, v.package_hash, v.manifest_blob_url, logger));
        })))
            .finally(() => (0, common_1.deleteFolderSync)(workDirectoryPath));
    }
    // eslint-disable-next-line max-lines-per-function
    releasePackage(appId, deploymentId, packageInfo, filePath, releaseUid, logger) {
        const { appVersion } = packageInfo;
        const versionInfo = (0, common_1.validatorVersion)(appVersion);
        if (!versionInfo[0]) {
            logger.debug(`releasePackage targetBinaryVersion ${appVersion} not support.`);
            return Promise.reject(new app_error_1.AppError(`targetBinaryVersion ${appVersion} not support.`));
        }
        const { description } = packageInfo; // 描述
        const { isDisabled } = packageInfo; // 是否立刻下载
        const { rollout } = packageInfo; // 灰度百分比
        const { isMandatory } = packageInfo; // 是否强制更新，无法跳过
        const tmpDir = os_1.default.tmpdir();
        const directoryPathParent = path_1.default.join(tmpDir, `codepuh_${(0, security_1.randToken)(32)}`);
        const directoryPath = path_1.default.join(directoryPathParent, 'current');
        logger.debug(`releasePackage generate an random dir path: ${directoryPath}`);
        return Promise.all([
            (0, qetag_1.qetag)(filePath, logger),
            (0, common_1.createEmptyFolder)(directoryPath).then(() => {
                return (0, common_1.unzipFile)(filePath, directoryPath, logger);
            }),
        ])
            .then(([blobHash]) => {
            return (0, security_1.uploadPackageType)(directoryPath).then((type) => {
                return apps_1.Apps.findByPk(appId).then((appInfo) => {
                    if (type > 0 && appInfo.os > 0 && appInfo.os !== type) {
                        const e = new app_error_1.AppError('it must be publish it by ios type');
                        logger.debug(e);
                        throw e;
                    }
                    else {
                        // 不验证
                        logger.debug(`Unknown package type:`, {
                            type,
                            os: appInfo.os,
                        });
                    }
                    return blobHash;
                });
            });
        })
            .then((blobHash) => {
            return datacenter_manager_1.dataCenterManager
                .storePackage(directoryPath, false, logger)
                .then((dataCenter) => {
                const { packageHash } = dataCenter;
                const manifestFile = dataCenter.manifestFilePath;
                return deployments_versions_1.DeploymentsVersions.findOne({
                    where: { deployment_id: deploymentId, app_version: appVersion },
                })
                    .then((deploymentsVersions) => {
                    if (!deploymentsVersions) {
                        return false;
                    }
                    return this.isMatchPackageHash(deploymentsVersions.get('current_package_id'), packageHash, logger);
                })
                    .then((isExist) => {
                    if (isExist) {
                        const e = new app_error_1.AppError("The uploaded package is identical to the contents of the specified deployment's current release.");
                        logger.debug(e.message);
                        throw e;
                    }
                    return (0, qetag_1.qetag)(manifestFile, logger);
                })
                    .then((manifestHash) => {
                    return Promise.all([
                        (0, storage_1.uploadFileToStorage)(manifestHash, manifestFile, logger),
                        (0, storage_1.uploadFileToStorage)(blobHash, filePath, logger),
                    ]).then(() => [packageHash, manifestHash, blobHash]);
                });
            });
        })
            .then(([packageHash, manifestHash, blobHash]) => {
            const stats = fs_1.default.statSync(filePath);
            const params = {
                releaseMethod: const_1.RELEASE_METHOD_UPLOAD,
                releaseUid,
                isMandatory: isMandatory ? const_1.IS_MANDATORY_YES : const_1.IS_MANDATORY_NO,
                isDisabled: isDisabled ? const_1.IS_DISABLED_YES : const_1.IS_DISABLED_NO,
                rollout,
                size: stats.size,
                description,
                min_version: versionInfo[1],
                max_version: versionInfo[2],
            };
            return this.createPackage(deploymentId, appVersion, packageHash, manifestHash, blobHash, params, logger);
        })
            .finally(() => (0, common_1.deleteFolderSync)(directoryPathParent));
    }
    modifyReleasePackage(packageId, params) {
        const appVersion = lodash_1.default.get(params, 'appVersion');
        const description = lodash_1.default.get(params, 'description');
        const isMandatory = lodash_1.default.get(params, 'isMandatory');
        const isDisabled = lodash_1.default.get(params, 'isDisabled');
        const rollout = lodash_1.default.get(params, 'rollout');
        return packages_1.Packages.findByPk(packageId)
            .then((packageInfo) => {
            if (!packageInfo) {
                throw new app_error_1.AppError(`packageInfo not found`);
            }
            if (!lodash_1.default.isNull(appVersion)) {
                const versionInfo = (0, common_1.validatorVersion)(appVersion);
                if (!versionInfo[0]) {
                    throw new app_error_1.AppError(`--targetBinaryVersion ${appVersion} not support.`);
                }
                return Promise.all([
                    deployments_versions_1.DeploymentsVersions.findOne({
                        where: {
                            deployment_id: packageInfo.deployment_id,
                            app_version: appVersion,
                        },
                    }),
                    deployments_versions_1.DeploymentsVersions.findByPk(packageInfo.deployment_version_id),
                ])
                    .then(([v1, v2]) => {
                    if (v1 && !lodash_1.default.eq(v1.id, v2.id)) {
                        throw new app_error_1.AppError(`${appVersion} already exist.`);
                    }
                    if (!v2) {
                        throw new app_error_1.AppError(`packages not found.`);
                    }
                    return deployments_versions_1.DeploymentsVersions.update({
                        app_version: appVersion,
                        min_version: versionInfo[1],
                        max_version: versionInfo[2],
                    }, { where: { id: v2.id } });
                })
                    .then(() => {
                    return packageInfo;
                });
            }
            return packageInfo;
        })
            .then((packageInfo) => {
            const newParams = {
                description: description || packageInfo.description,
            };
            if (lodash_1.default.isInteger(rollout)) {
                newParams.rollout = rollout;
            }
            if (lodash_1.default.isBoolean(isMandatory)) {
                newParams.is_mandatory = isMandatory ? const_1.IS_MANDATORY_YES : const_1.IS_MANDATORY_NO;
            }
            if (lodash_1.default.isBoolean(isDisabled)) {
                newParams.is_disabled = isDisabled ? const_1.IS_DISABLED_YES : const_1.IS_DISABLED_NO;
            }
            return packages_1.Packages.update(newParams, { where: { id: packageId } });
        });
    }
    // eslint-disable-next-line max-lines-per-function
    promotePackage(sourceDeploymentInfo, destDeploymentInfo, params, logger) {
        const appVersion = lodash_1.default.get(params, 'appVersion', null);
        const label = lodash_1.default.get(params, 'label', null);
        return new Promise((resolve, reject) => {
            if (label) {
                packages_1.Packages.findOne({
                    where: { deployment_id: sourceDeploymentInfo.id, label },
                })
                    .then((sourcePack) => {
                    if (!sourcePack) {
                        throw new app_error_1.AppError('label does not exist.');
                    }
                    return deployments_versions_1.DeploymentsVersions.findByPk(sourcePack.deployment_version_id).then((deploymentsVersions) => {
                        if (!deploymentsVersions) {
                            throw new app_error_1.AppError('deploymentsVersions does not exist.');
                        }
                        resolve([sourcePack, deploymentsVersions]);
                    });
                })
                    .catch((e) => {
                    reject(e);
                });
                return;
            }
            const lastDeploymentVersionId = lodash_1.default.get(sourceDeploymentInfo, 'last_deployment_version_id', 0);
            if (lodash_1.default.lte(lastDeploymentVersionId, 0)) {
                throw new app_error_1.AppError(`does not exist last_deployment_version_id.`);
            }
            deployments_versions_1.DeploymentsVersions.findByPk(lastDeploymentVersionId)
                .then((deploymentsVersions) => {
                const sourcePackId = lodash_1.default.get(deploymentsVersions, 'current_package_id', 0);
                if (lodash_1.default.lte(sourcePackId, 0)) {
                    throw new app_error_1.AppError(`packageInfo not found.`);
                }
                return packages_1.Packages.findByPk(sourcePackId).then((sourcePack) => {
                    if (!sourcePack) {
                        throw new app_error_1.AppError(`packageInfo not found.`);
                    }
                    resolve([sourcePack, deploymentsVersions]);
                });
            })
                .catch((e) => {
                reject(e);
            });
        })
            .then(([sourcePack, deploymentsVersions]) => {
            const appFinalVersion = appVersion || deploymentsVersions.app_version;
            logger.debug('sourcePack', sourcePack);
            logger.debug('deploymentsVersions', deploymentsVersions);
            logger.debug('appFinalVersion', appFinalVersion);
            return deployments_versions_1.DeploymentsVersions.findOne({
                where: {
                    deployment_id: destDeploymentInfo.id,
                    app_version: appFinalVersion,
                },
            })
                .then((destDeploymentsVersions) => {
                if (!destDeploymentsVersions) {
                    return false;
                }
                return this.isMatchPackageHash(destDeploymentsVersions.get('current_package_id'), sourcePack.package_hash, logger);
            })
                .then((isExist) => {
                if (isExist) {
                    throw new app_error_1.AppError("The uploaded package is identical to the contents of the specified deployment's current release.");
                }
                return [sourcePack, appFinalVersion];
            });
        })
            .then(([sourcePack, appFinalVersion]) => {
            const versionInfo = (0, common_1.validatorVersion)(appFinalVersion);
            if (!versionInfo[0]) {
                logger.debug(`targetBinaryVersion ${appVersion} not support.`);
                throw new app_error_1.AppError(`targetBinaryVersion ${appVersion} not support.`);
            }
            const createParams = {
                releaseMethod: const_1.RELEASE_METHOD_PROMOTE,
                releaseUid: params.promoteUid || 0,
                rollout: params.rollout || 100,
                size: sourcePack.size,
                description: params.description || sourcePack.description,
                originalLabel: sourcePack.label,
                originalDeployment: sourceDeploymentInfo.name,
                min_version: versionInfo[1],
                max_version: versionInfo[2],
                isMandatory: 0,
                isDisabled: 0,
            };
            if (lodash_1.default.isBoolean(params.isMandatory)) {
                createParams.isMandatory = params.isMandatory
                    ? const_1.IS_MANDATORY_YES
                    : const_1.IS_MANDATORY_NO;
            }
            else {
                createParams.isMandatory = sourcePack.is_mandatory;
            }
            if (lodash_1.default.isBoolean(params.isDisabled)) {
                createParams.isDisabled = params.isDisabled ? const_1.IS_DISABLED_YES : const_1.IS_DISABLED_NO;
            }
            else {
                createParams.isDisabled = sourcePack.is_disabled;
            }
            return this.createPackage(destDeploymentInfo.id, appFinalVersion, sourcePack.package_hash, sourcePack.manifest_blob_url, sourcePack.blob_url, createParams, logger);
        });
    }
    rollbackPackage(deploymentVersionId, targetLabel, rollbackUid, logger) {
        return deployments_versions_1.DeploymentsVersions.findByPk(deploymentVersionId).then((deploymentsVersions) => {
            if (!deploymentsVersions) {
                throw new app_error_1.AppError('您之前还没有发布过版本');
            }
            return packages_1.Packages.findByPk(deploymentsVersions.current_package_id)
                .then((currentPackageInfo) => {
                if (targetLabel) {
                    return packages_1.Packages.findAll({
                        where: {
                            deployment_version_id: deploymentVersionId,
                            label: targetLabel,
                        },
                        limit: 1,
                    }).then((rollbackPackageInfos) => {
                        return [currentPackageInfo, rollbackPackageInfos];
                    });
                }
                return this.getCanRollbackPackages(deploymentVersionId).then((rollbackPackageInfos) => {
                    return [currentPackageInfo, rollbackPackageInfos];
                });
            })
                .then(([currentPackageInfo, rollbackPackageInfos]) => {
                if (currentPackageInfo && rollbackPackageInfos.length > 0) {
                    for (let i = rollbackPackageInfos.length - 1; i >= 0; i -= 1) {
                        if (rollbackPackageInfos[i].package_hash !==
                            currentPackageInfo.package_hash) {
                            return rollbackPackageInfos[i];
                        }
                    }
                }
                throw new app_error_1.AppError('没有可供回滚的版本');
            })
                .then((rollbackPackage) => {
                const params = {
                    releaseMethod: 'Rollback',
                    releaseUid: rollbackUid,
                    isMandatory: rollbackPackage.is_mandatory,
                    isDisabled: rollbackPackage.is_disabled,
                    rollout: rollbackPackage.rollout,
                    size: rollbackPackage.size,
                    description: rollbackPackage.description,
                    originalLabel: rollbackPackage.label,
                    originalDeployment: '',
                    min_version: deploymentsVersions.min_version,
                    max_version: deploymentsVersions.max_version,
                };
                return this.createPackage(deploymentsVersions.deployment_id, deploymentsVersions.app_version, rollbackPackage.package_hash, rollbackPackage.manifest_blob_url, rollbackPackage.blob_url, params, logger);
            });
        });
    }
    getCanRollbackPackages(deploymentVersionId) {
        return packages_1.Packages.findAll({
            where: {
                deployment_version_id: deploymentVersionId,
                release_method: {
                    [sequelize_1.Op.in]: [const_1.RELEASE_METHOD_UPLOAD, const_1.RELEASE_METHOD_PROMOTE],
                },
            },
            order: [['id', 'desc']],
            limit: 2,
        });
    }
}
exports.packageManager = new PackageManager();
