"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.calcAllFileSha256 = exports.uploadPackageType = exports.packageHashSync = exports.parseToken = exports.randToken = exports.passwordVerifySync = exports.passwordHashSync = exports.md5 = void 0;
const crypto_1 = __importDefault(require("crypto"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const bcryptjs_1 = __importDefault(require("bcryptjs"));
const kv_logger_1 = require("kv-logger");
const lodash_1 = __importDefault(require("lodash"));
const rand_token_1 = require("rand-token");
const recursive_readdir_1 = __importDefault(require("recursive-readdir"));
const slash_1 = __importDefault(require("slash"));
const app_error_1 = require("../app-error");
const const_1 = require("../const");
function md5(str) {
    const md5sum = crypto_1.default.createHash('md5');
    md5sum.update(str);
    return md5sum.digest('hex');
}
exports.md5 = md5;
function passwordHashSync(password) {
    return bcryptjs_1.default.hashSync(password, bcryptjs_1.default.genSaltSync(12));
}
exports.passwordHashSync = passwordHashSync;
function passwordVerifySync(password, hash) {
    return bcryptjs_1.default.compareSync(password, hash);
}
exports.passwordVerifySync = passwordVerifySync;
const randTokenGen = (0, rand_token_1.generator)({
    chars: '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ',
    source: 'crypto',
});
function randToken(num) {
    return randTokenGen.generate(num);
}
exports.randToken = randToken;
function parseToken(token) {
    return { identical: token.substring(token.length - 9), token: token.substring(0, 28) };
}
exports.parseToken = parseToken;
function fileSha256(file) {
    return new Promise((resolve, reject) => {
        const rs = fs_1.default.createReadStream(file);
        const hash = crypto_1.default.createHash('sha256');
        rs.on('data', hash.update.bind(hash));
        rs.on('error', (e) => {
            reject(e);
        });
        rs.on('end', () => {
            resolve(hash.digest('hex'));
        });
    });
}
function stringSha256Sync(contents) {
    const sha256 = crypto_1.default.createHash('sha256');
    sha256.update(contents);
    return sha256.digest('hex');
}
function sortJsonToArr(json) {
    const rs = [];
    lodash_1.default.forIn(json, (value, key) => {
        rs.push({ path: key, hash: value });
    });
    return lodash_1.default.sortBy(rs, (o) => o.path);
}
// some files are ignored in calc hash in client sdk
// https://github.com/Microsoft/react-native-code-push/pull/974/files#diff-21b650f88429c071b217d46243875987R15
function isHashIgnored(relativePath) {
    if (!relativePath) {
        return true;
    }
    const IgnoreMacOSX = '__MACOSX/';
    const IgnoreDSStore = '.DS_Store';
    return (relativePath.startsWith(IgnoreMacOSX) ||
        relativePath === IgnoreDSStore ||
        relativePath.endsWith(IgnoreDSStore));
}
function isPackageHashIgnored(relativePath) {
    if (!relativePath) {
        return true;
    }
    // .codepushrelease contains code sign JWT
    // it should be ignored in package hash but need to be included in package manifest
    const IgnoreCodePushMetadata = '.codepushrelease';
    return (relativePath === IgnoreCodePushMetadata ||
        relativePath.endsWith(IgnoreCodePushMetadata) ||
        isHashIgnored(relativePath));
}
function packageHashSync(jsonData) {
    const sortedArr = sortJsonToArr(jsonData);
    const manifestData = lodash_1.default.filter(sortedArr, (v) => {
        return !isPackageHashIgnored(v.path);
    }).map((v) => {
        return `${v.path}:${v.hash}`;
    });
    let manifestString = JSON.stringify(manifestData.sort());
    manifestString = lodash_1.default.replace(manifestString, /\\\//g, '/');
    kv_logger_1.logger.debug('packageHashSync manifestString', {
        manifestString,
    });
    return stringSha256Sync(manifestString);
}
exports.packageHashSync = packageHashSync;
function sha256AllFiles(files) {
    return new Promise((resolve) => {
        const results = {};
        const { length } = files;
        let count = 0;
        files.forEach((file) => {
            fileSha256(file).then((hash) => {
                results[file] = hash;
                count += 1;
                if (count === length) {
                    resolve(results);
                }
            });
        });
    });
}
function uploadPackageType(directoryPath) {
    return new Promise((resolve, reject) => {
        (0, recursive_readdir_1.default)(directoryPath, (err, files) => {
            if (err) {
                kv_logger_1.logger.error(new app_error_1.AppError(err.message));
                reject(new app_error_1.AppError(err.message));
            }
            else if (files.length === 0) {
                kv_logger_1.logger.debug(`uploadPackageType empty files`);
                reject(new app_error_1.AppError('empty files'));
            }
            else {
                const aregex = /android\.bundle/;
                const aregexIOS = /main\.jsbundle/;
                let packageType = 0;
                lodash_1.default.forIn(files, (value) => {
                    if (aregex.test(value)) {
                        packageType = const_1.ANDROID;
                        return false;
                    }
                    if (aregexIOS.test(value)) {
                        packageType = const_1.IOS;
                        return false;
                    }
                    return undefined;
                });
                kv_logger_1.logger.debug(`uploadPackageType packageType: ${packageType}`);
                resolve(packageType);
            }
        });
    });
}
exports.uploadPackageType = uploadPackageType;
function calcAllFileSha256(directoryPath) {
    return new Promise((resolve, reject) => {
        (0, recursive_readdir_1.default)(directoryPath, (error, files) => {
            if (error) {
                kv_logger_1.logger.error(error);
                reject(new app_error_1.AppError(error.message));
            }
            else {
                // filter files that should be ignored
                // eslint-disable-next-line no-param-reassign
                files = files.filter((file) => {
                    const relative = path_1.default.relative(directoryPath, file);
                    return !isHashIgnored(relative);
                });
                if (files.length === 0) {
                    kv_logger_1.logger.debug(`calcAllFileSha256 empty files in directory`, { directoryPath });
                    reject(new app_error_1.AppError('empty files'));
                }
                else {
                    sha256AllFiles(files).then((results) => {
                        const data = {};
                        lodash_1.default.forIn(results, (value, key) => {
                            let relativePath = path_1.default.relative(directoryPath, key);
                            relativePath = (0, slash_1.default)(relativePath);
                            data[relativePath] = value;
                        });
                        kv_logger_1.logger.debug(`calcAllFileSha256 files:`, data);
                        resolve(data);
                    });
                }
            }
        });
    });
}
exports.calcAllFileSha256 = calcAllFileSha256;
