"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.uploadFileToStorage = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const aliyun_oss_upload_stream_1 = __importDefault(require("aliyun-oss-upload-stream"));
const aliyun_sdk_1 = __importDefault(require("aliyun-sdk"));
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const cos_nodejs_sdk_v5_1 = __importDefault(require("cos-nodejs-sdk-v5"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const lodash_1 = __importDefault(require("lodash"));
const qiniu_1 = __importDefault(require("qiniu"));
const app_error_1 = require("../app-error");
const config_1 = require("../config");
function uploadFileToLocal(key, filePath, logger) {
    return new Promise((resolve, reject) => {
        logger.info(`try uploadFileToLocal`, {
            key,
        });
        const storageDir = lodash_1.default.get(config_1.config, 'local.storageDir');
        if (!storageDir) {
            throw new app_error_1.AppError('please set config local storageDir');
        }
        if (key.length < 3) {
            logger.error(`generate key is too short, key value:${key}`);
            throw new app_error_1.AppError('generate key is too short.');
        }
        try {
            logger.debug(`uploadFileToLocal check directory ${storageDir} fs.W_OK`);
            fs_1.default.accessSync(storageDir, fs_1.default.constants.W_OK);
            logger.debug(`uploadFileToLocal directory ${storageDir} fs.W_OK is ok`);
        }
        catch (err) {
            throw new app_error_1.AppError(err);
        }
        const subDir = key.substring(0, 2).toLowerCase();
        const finalDir = path_1.default.join(storageDir, subDir);
        const fileName = path_1.default.join(finalDir, key);
        if (fs_1.default.existsSync(fileName)) {
            logger.info(`uploadFileToLocal file exists, skip copy`, {
                key,
            });
            resolve();
            return;
        }
        let stats = fs_1.default.statSync(storageDir);
        if (!stats.isDirectory()) {
            throw new app_error_1.AppError(`${storageDir} must be directory`);
        }
        if (!fs_1.default.existsSync(`${finalDir}`)) {
            fs_1.default.mkdirSync(`${finalDir}`);
            logger.info(`uploadFileToLocal mkdir:${finalDir}`, {
                key,
            });
        }
        try {
            fs_1.default.accessSync(filePath, fs_1.default.constants.R_OK);
        }
        catch (err) {
            throw new app_error_1.AppError(err);
        }
        stats = fs_1.default.statSync(filePath);
        if (!stats.isFile()) {
            throw new app_error_1.AppError(`${filePath} must be file`);
        }
        fs_extra_1.default.copy(filePath, fileName, (err) => {
            if (err) {
                reject(new app_error_1.AppError(err));
                return;
            }
            logger.info(`uploadFileToLocal copy file success.`, {
                key,
            });
            resolve();
        });
    });
}
function uploadFileToS3(key, filePath, logger) {
    return new Promise((resolve, reject) => {
        logger.info('try uploadFileToS3', { key });
        aws_sdk_1.default.config.update({
            accessKeyId: lodash_1.default.get(config_1.config, 's3.accessKeyId'),
            secretAccessKey: lodash_1.default.get(config_1.config, 's3.secretAccessKey'),
            sessionToken: lodash_1.default.get(config_1.config, 's3.sessionToken'),
            region: lodash_1.default.get(config_1.config, 's3.region'),
        });
        const s3 = new aws_sdk_1.default.S3();
        fs_1.default.readFile(filePath, (err, data) => {
            if (err) {
                reject(new app_error_1.AppError(err));
                return;
            }
            s3.upload({
                Key: key,
                Body: data,
                ACL: 'public-read',
                Bucket: lodash_1.default.get(config_1.config, 's3.bucketName'),
            }, (error) => {
                if (error) {
                    reject(new app_error_1.AppError(error));
                }
                else {
                    logger.info('uploadFileToS3 success', { key });
                    resolve();
                }
            });
        });
    });
}
function uploadFileToOSS(key, filePath, logger) {
    logger.info('try uploadFileToOSS', { key });
    const ossStream = (0, aliyun_oss_upload_stream_1.default)(new aliyun_sdk_1.default.OSS({
        accessKeyId: lodash_1.default.get(config_1.config, 'oss.accessKeyId'),
        secretAccessKey: lodash_1.default.get(config_1.config, 'oss.secretAccessKey'),
        endpoint: lodash_1.default.get(config_1.config, 'oss.endpoint'),
        apiVersion: '2013-10-15',
    }));
    if (!lodash_1.default.isEmpty(lodash_1.default.get(config_1.config, 'oss.prefix', ''))) {
        // eslint-disable-next-line no-param-reassign
        key = `${lodash_1.default.get(config_1.config, 'oss.prefix')}/${key}`;
    }
    const upload = ossStream.upload({
        Bucket: lodash_1.default.get(config_1.config, 'oss.bucketName'),
        Key: key,
    });
    return new Promise((resolve, reject) => {
        upload.on('error', (error) => {
            reject(new app_error_1.AppError(JSON.stringify(error)));
        });
        upload.on('uploaded', () => {
            logger.info('uploadFileToOSS success', { key });
            resolve();
        });
        fs_1.default.createReadStream(filePath).pipe(upload);
    });
}
function getUploadTokenQiniu(mac, bucket, key) {
    const options = {
        scope: `${bucket}:${key}`,
    };
    const putPolicy = new qiniu_1.default.rs.PutPolicy(options);
    return putPolicy.uploadToken(mac);
}
function uploadFileToQiniu(key, filePath, logger) {
    return new Promise((resolve, reject) => {
        logger.info('try uploadFileToQiniu', { key });
        const accessKey = lodash_1.default.get(config_1.config, 'qiniu.accessKey');
        const secretKey = lodash_1.default.get(config_1.config, 'qiniu.secretKey');
        const bucket = lodash_1.default.get(config_1.config, 'qiniu.bucketName', '');
        const mac = new qiniu_1.default.auth.digest.Mac(accessKey, secretKey);
        const conf = new qiniu_1.default.conf.Config();
        const bucketManager = new qiniu_1.default.rs.BucketManager(mac, conf);
        bucketManager.stat(bucket, key, (respErr, respBody, respInfo) => {
            if (respErr) {
                reject(new app_error_1.AppError(respErr.message));
                return;
            }
            if (respInfo.statusCode === 200) {
                logger.info('uploadFileToQiniu file exists, skip upload', { key });
                resolve();
                return;
            }
            let uploadToken;
            try {
                uploadToken = getUploadTokenQiniu(mac, bucket, key);
            }
            catch (e) {
                reject(new app_error_1.AppError(e.message));
                return;
            }
            const formUploader = new qiniu_1.default.form_up.FormUploader(conf);
            const putExtra = new qiniu_1.default.form_up.PutExtra();
            formUploader.putFile(uploadToken, key, filePath, putExtra, (resErr, resBody, resInfo) => {
                if (resErr) {
                    // 上传失败， 处理返回代码
                    return reject(new app_error_1.AppError(resErr));
                }
                // 上传成功， 处理返回值
                if (resInfo.statusCode === 200) {
                    logger.info('uploadFileToQiniu success', { key });
                    return resolve();
                }
                return reject(new app_error_1.AppError(resBody.error));
            });
        });
    });
}
function uploadFileToTencentCloud(key, filePath, logger) {
    return new Promise((resolve, reject) => {
        logger.info('try uploadFileToTencentCloud', { key });
        const cosIn = new cos_nodejs_sdk_v5_1.default({
            SecretId: lodash_1.default.get(config_1.config, 'tencentcloud.accessKeyId'),
            SecretKey: lodash_1.default.get(config_1.config, 'tencentcloud.secretAccessKey'),
        });
        let cosFolder=lodash_1.default.get(config_1.config, 'tencentcloud.folder')
        logger.info('tencentcloud.folder1===',lodash_1.default.get(config_1.config, 'tencentcloud.folder'))

        if(cosFolder.substr(-1,1) !== '/'){
            cosFolder = cosFolder+'/'
        }
        logger.info('tencentcloud.folder===',cosFolder)
        cosIn.sliceUploadFile({
            Bucket: lodash_1.default.get(config_1.config, 'tencentcloud.bucketName'),
            Region: lodash_1.default.get(config_1.config, 'tencentcloud.region'),
            Key: cosFolder+key,
            FilePath: filePath,
        }, (err,data) => {
            if (err) {
                reject(new app_error_1.AppError(err.message));
            }
            else {
                logger.info(`uploadFileToTencentCloud success===Location=${data.Location}=ETag=${data.ETag}==versionId=${data.VersionId}`, { data });
                resolve(data.Location);
            }
        });
    });
}
function uploadFileToStorage(key, filePath, logger) {
    const { storageType } = config_1.config.common;
    switch (storageType) {
        case 'local':
            return uploadFileToLocal(key, filePath, logger);
        case 's3':
            return uploadFileToS3(key, filePath, logger);
        case 'oss':
            return uploadFileToOSS(key, filePath, logger);
        case 'qiniu':
            return uploadFileToQiniu(key, filePath, logger);
        case 'tencentcloud':
            return uploadFileToTencentCloud(key, filePath, logger);
        default:
            throw new app_error_1.AppError(`${storageType} storageType does not support.`);
    }
}
exports.uploadFileToStorage = uploadFileToStorage;
