"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.accessKeysRouter = void 0;
const express_1 = __importDefault(require("express"));
const lodash_1 = __importDefault(require("lodash"));
const moment_1 = __importDefault(require("moment"));
const app_error_1 = require("../core/app-error");
const middleware_1 = require("../core/middleware");
const account_manager_1 = require("../core/services/account-manager");
const security_1 = require("../core/utils/security");
const user_tokens_1 = require("../models/user_tokens");
exports.accessKeysRouter = express_1.default.Router();
exports.accessKeysRouter.get('/', middleware_1.checkToken, (req, res, next) => {
    const { logger } = req;
    const uid = req.users.id;
    logger.info('try get acceesKeys', { uid });
    account_manager_1.accountManager
        .getAllAccessKeyByUid(uid)
        .then((accessKeys) => {
        logger.info('get acceesKeys success', { uid });
        res.send({ accessKeys });
    })
        .catch((e) => {
        next(e);
    });
});
exports.accessKeysRouter.post('/', middleware_1.checkToken, (req, res, next) => {
    const { logger, body } = req;
    const uid = req.users.id;
    const createdBy = lodash_1.default.trim(body.createdBy);
    const friendlyName = lodash_1.default.trim(body.friendlyName);
    const ttl = parseInt(body.ttl, 10);
    const description = lodash_1.default.trim(body.description);
    logger.info('try to generate access key', {
        uid,
        name: friendlyName,
        body: JSON.stringify(body),
    });
    return account_manager_1.accountManager
        .isExsitAccessKeyName(uid, friendlyName)
        .then((data) => {
        if (!lodash_1.default.isEmpty(data)) {
            throw new app_error_1.AppError(`The access key "${friendlyName}"  already exists.`);
        }
    })
        .then(() => {
        const { identical } = req.users;
        const newAccessKey = (0, security_1.randToken)(28).concat(identical);
        return account_manager_1.accountManager.createAccessKey(uid, newAccessKey, ttl, friendlyName, createdBy, description);
    })
        .then((newToken) => {
        const info = {
            name: newToken.tokens,
            createdTime: (0, moment_1.default)(newToken.created_at).valueOf(),
            createdBy: newToken.created_by,
            expires: (0, moment_1.default)(newToken.expires_at).valueOf(),
            description: newToken.description,
            friendlyName: newToken.name,
        };
        logger.info('create access key success', {
            uid,
            name: newToken.name,
        });
        res.send({ accessKey: info });
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('create access key failed', {
                uid,
                name: friendlyName,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.accessKeysRouter.delete('/:name', middleware_1.checkToken, (req, res, next) => {
    const { logger, params } = req;
    const name = lodash_1.default.trim(params.name);
    const uid = req.users.id;
    logger.info('try to delete access key', { uid, name });
    return user_tokens_1.UserTokens.destroy({ where: { name, uid } })
        .then(() => {
        logger.info('delete acceesKey success', { uid, name });
        res.send({ friendlyName: name });
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('delete acceesKey failed', { uid, name });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
