"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.appsRouter = void 0;
/* eslint-disable max-lines */
const express_1 = __importDefault(require("express"));
const lodash_1 = __importDefault(require("lodash"));
const validator_1 = __importDefault(require("validator"));
const app_error_1 = require("../core/app-error");
const config_1 = require("../core/config");
const const_1 = require("../core/const");
const middleware_1 = require("../core/middleware");
const account_manager_1 = require("../core/services/account-manager");
const app_manager_1 = require("../core/services/app-manager");
const client_manager_1 = require("../core/services/client-manager");
const collaborators_manager_1 = require("../core/services/collaborators-manager");
const deployments_manager_1 = require("../core/services/deployments-manager");
const package_manager_1 = require("../core/services/package-manager");
const common_1 = require("../core/utils/common");
function delay(ms) {
    return new Promise((resolve) => {
        setTimeout(resolve, ms);
    });
}
exports.appsRouter = express_1.default.Router();
exports.appsRouter.get('/', middleware_1.checkToken, (req, res, next) => {
    const { logger } = req;
    const uid = req.users.id;
    logger.info('try list apps', { uid });
    app_manager_1.appManager
        .listApps(uid)
        .then((data) => {
        logger.info('list apps success', { uid });
        res.send({ apps: data });
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('list apps failed', { uid, error: e.message });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.get('/:appName/deployments', middleware_1.checkToken, (req, res, next) => {
    const { logger, params } = req;
    const uid = req.users.id;
    const appName = lodash_1.default.trim(params.appName);
    logger.info('try list deployments', {
        uid,
        appName,
    });
    account_manager_1.accountManager
        .collaboratorCan(uid, appName, logger)
        .then((col) => {
        return deployments_manager_1.deploymentsManager.listDeloyments(col.appid);
    })
        .then((data) => {
        logger.info('list deployments success', {
            uid,
            appName,
        });
        res.send({ deployments: data });
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('list deployments failed', {
                uid,
                appName,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.get('/:appName/deployments/:deploymentName', middleware_1.checkToken, (req, res, next) => {
    const { logger, params } = req;
    const uid = req.users.id;
    const appName = lodash_1.default.trim(params.appName);
    const deploymentName = lodash_1.default.trim(params.deploymentName);
    logger.info('try to get deployment', {
        uid,
        appName,
        deploymentName,
    });
    account_manager_1.accountManager
        .collaboratorCan(uid, appName, logger)
        .then((col) => {
        return deployments_manager_1.deploymentsManager.findDeloymentByName(deploymentName, col.appid, logger);
    })
        .then((deploymentInfo) => {
        if (lodash_1.default.isEmpty(deploymentInfo)) {
            throw new app_error_1.AppError('does not find the deployment');
        }
        logger.info('get deployment success', {
            uid,
            appName,
            deploymentName,
        });
        // TODO: check if this works as expected
        res.send({ deployment: deployments_manager_1.deploymentsManager.listDeloyment(deploymentInfo) });
        return true;
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('get deployment failed', {
                uid,
                appName,
                deploymentName,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.post('/:appName/deployments', middleware_1.checkToken, (req, res, next) => {
    const { logger, params, body } = req;
    const uid = req.users.id;
    const appName = lodash_1.default.trim(params.appName);
    const { name } = body;
    logger.info('try to create deployment', {
        uid,
        appName,
        deploymentName: name,
    });
    account_manager_1.accountManager
        .ownerCan(uid, appName, logger)
        .then((col) => {
        return deployments_manager_1.deploymentsManager.addDeloyment(name, col.appid, uid);
    })
        .then((data) => {
        logger.info('create deployment success', {
            uid,
            appName,
            deploymentName: data.name,
        });
        res.send({ deployment: { name: data.name, key: data.deployment_key } });
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('create deployment failed', {
                uid,
                appName,
                deploymentName: name,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.get('/:appName/deployments/:deploymentName/metrics', middleware_1.checkToken, (req, res, next) => {
    const { logger, params } = req;
    const uid = req.users.id;
    const appName = lodash_1.default.trim(params.appName);
    const deploymentName = lodash_1.default.trim(params.deploymentName);
    logger.info('try to get deployment metrics', {
        uid,
        appName,
        deploymentName,
    });
    account_manager_1.accountManager
        .collaboratorCan(uid, appName, logger)
        .then((col) => {
        return deployments_manager_1.deploymentsManager
            .findDeloymentByName(deploymentName, col.appid, logger)
            .then((deploymentInfo) => {
            if (lodash_1.default.isEmpty(deploymentInfo)) {
                throw new app_error_1.AppError('does not find the deployment');
            }
            return deploymentInfo;
        });
    })
        .then((deploymentInfo) => {
        return deployments_manager_1.deploymentsManager.getAllPackageIdsByDeploymentsId(deploymentInfo.id);
    })
        .then((packagesInfos) => {
        return packagesInfos.reduce((prev, v) => {
            return prev.then((result) => {
                return package_manager_1.packageManager
                    .getMetricsbyPackageId(v.get('id'))
                    .then((metrics) => {
                    if (metrics) {
                        result[v.get('label')] = {
                            active: metrics.get('active'),
                            downloaded: metrics.get('downloaded'),
                            failed: metrics.get('failed'),
                            installed: metrics.get('installed'),
                        };
                    }
                    return result;
                });
            });
        }, Promise.resolve({}));
    })
        .then((rs) => {
        logger.info('get deployment metrics success', {
            uid,
            appName,
            deploymentName,
        });
        res.send({ metrics: rs });
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('get deployment metrics failed', {
                uid,
                appName,
                deploymentName,
                error: e.message,
            });
            res.send({ metrics: null });
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.get('/:appName/deployments/:deploymentName/history', middleware_1.checkToken, (req, res, next) => {
    const { logger, params } = req;
    const uid = req.users.id;
    const appName = lodash_1.default.trim(params.appName);
    const deploymentName = lodash_1.default.trim(params.deploymentName);
    logger.info('try to get deployment history', {
        uid,
        appName,
        deploymentName,
    });
    account_manager_1.accountManager
        .collaboratorCan(uid, appName, logger)
        .then((col) => {
        return deployments_manager_1.deploymentsManager
            .findDeloymentByName(deploymentName, col.appid, logger)
            .then((deploymentInfo) => {
            if (lodash_1.default.isEmpty(deploymentInfo)) {
                throw new app_error_1.AppError('does not find the deployment');
            }
            return deploymentInfo;
        });
    })
        .then((deploymentInfo) => {
        return deployments_manager_1.deploymentsManager.getDeploymentHistory(deploymentInfo.id);
    })
        .then((rs) => {
        logger.info('get deployment history success', {
            uid,
            appName,
            deploymentName,
        });
        res.send({ history: lodash_1.default.pull(rs, null) });
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('get deployment history failed', {
                uid,
                appName,
                deploymentName,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.delete('/:appName/deployments/:deploymentName/history', middleware_1.checkToken, (req, res, next) => {
    const { logger, params } = req;
    const uid = req.users.id;
    const appName = lodash_1.default.trim(params.appName);
    const deploymentName = lodash_1.default.trim(params.deploymentName);
    logger.info('try to delete deployment history', {
        uid,
        appName,
        deploymentName,
    });
    account_manager_1.accountManager
        .ownerCan(uid, appName, logger)
        .then((col) => {
        return deployments_manager_1.deploymentsManager
            .findDeloymentByName(deploymentName, col.appid, logger)
            .then((deploymentInfo) => {
            if (lodash_1.default.isEmpty(deploymentInfo)) {
                throw new app_error_1.AppError('does not find the deployment');
            }
            return deploymentInfo;
        });
    })
        .then((deploymentInfo) => {
        return deployments_manager_1.deploymentsManager.deleteDeploymentHistory(deploymentInfo.id);
    })
        .then(() => {
        logger.info('delete deployment history success', {
            uid,
            appName,
            deploymentName,
        });
        res.send('ok');
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('delete deployment history failed', {
                uid,
                appName,
                deploymentName,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.patch('/:appName/deployments/:deploymentName', middleware_1.checkToken, (req, res, next) => {
    const { logger, params, body } = req;
    const { name } = body;
    const appName = lodash_1.default.trim(params.appName);
    const deploymentName = lodash_1.default.trim(params.deploymentName);
    const uid = req.users.id;
    logger.info('try to update deployment', {
        uid,
        appName,
        deploymentName,
        newName: name,
    });
    account_manager_1.accountManager
        .ownerCan(uid, appName, logger)
        .then((col) => {
        return deployments_manager_1.deploymentsManager.renameDeloymentByName(deploymentName, col.appid, name);
    })
        .then((data) => {
        logger.info('update deployment success', {
            uid,
            appName,
            deploymentName,
            newName: name,
        });
        res.send({ deployment: data });
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('update deployment failed', {
                uid,
                appName,
                deploymentName,
                newName: name,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.delete('/:appName/deployments/:deploymentName', middleware_1.checkToken, (req, res, next) => {
    const { logger, params } = req;
    const appName = lodash_1.default.trim(params.appName);
    const deploymentName = lodash_1.default.trim(params.deploymentName);
    const uid = req.users.id;
    logger.info('try to delete deployment', {
        uid,
        appName,
        deploymentName,
    });
    account_manager_1.accountManager
        .ownerCan(uid, appName, logger)
        .then((col) => {
        return deployments_manager_1.deploymentsManager.deleteDeloymentByName(deploymentName, col.appid);
    })
        .then((data) => {
        logger.info('delete deployment success', {
            uid,
            appName,
            deploymentName,
        });
        res.send({ deployment: data });
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('delete deployment failed', {
                uid,
                appName,
                deploymentName,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.post('/:appName/deployments/:deploymentName/release', middleware_1.checkToken, 
// eslint-disable-next-line max-lines-per-function
(req, res, next) => {
    const { logger, params } = req;
    const appName = lodash_1.default.trim(params.appName);
    const deploymentName = lodash_1.default.trim(params.deploymentName);
    const uid = req.users.id;
    logger.info('try to release', {
        uid,
        appName,
        deploymentName,
    });
    account_manager_1.accountManager
        .collaboratorCan(uid, appName, logger)
        .then((col) => {
        logger.info('release user check ok', {
            uid,
            appName,
            deploymentName,
        });
        return deployments_manager_1.deploymentsManager
            .findDeloymentByName(deploymentName, col.appid, logger)
            .then((deploymentInfo) => {
            if (lodash_1.default.isEmpty(deploymentInfo)) {
                throw new app_error_1.AppError('does not find the deployment');
            }
            logger.info('release deployment check ok', {
                uid,
                appName,
                deploymentName,
            });
            return package_manager_1.packageManager
                .parseReqFile(req, logger)
                .then((data) => {
                if (data.package.mimetype !== 'application/zip') {
                    throw new app_error_1.AppError(`upload file type is invalidate: ${data.package.mimetype}`);
                }
                logger.info('release packagee parse ok', {
                    uid,
                    appName,
                    deploymentName,
                });
                return package_manager_1.packageManager
                    .releasePackage(deploymentInfo.appid, deploymentInfo.id, data.packageInfo, data.package.filepath, uid, logger)
                    .finally(() => {
                    (0, common_1.deleteFolderSync)(data.package.filepath);
                });
            })
                .then((packages) => {
                if (packages) {
                    delay(1000).then(() => {
                        package_manager_1.packageManager
                            .createDiffPackagesByLastNums(deploymentInfo.appid, packages, config_1.config.common.diffNums, logger)
                            .catch((e) => {
                            logger.error(e);
                        });
                    });
                }
                // clear cache if exists.
                if (config_1.config.common.updateCheckCache) {
                    delay(2500).then(() => {
                        client_manager_1.clientManager.clearUpdateCheckCache(deploymentInfo.deployment_key, '*', '*', '*', logger);
                    });
                }
                return null;
            });
        });
    })
        .then(() => {
        logger.info('release success', {
            uid,
            appName,
            deploymentName,
        });
        res.send('{"msg": "succeed"}');
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('release failed', {
                uid,
                appName,
                deploymentName,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.patch('/:appName/deployments/:deploymentName/release', middleware_1.checkToken, 
// eslint-disable-next-line max-lines-per-function
(req, res, next) => {
    const { logger, params, body } = req;
    const appName = lodash_1.default.trim(params.appName);
    const deploymentName = lodash_1.default.trim(params.deploymentName);
    const uid = req.users.id;
    logger.info('try modify release package', {
        uid,
        appName,
        deploymentName,
        body: JSON.stringify(body),
    });
    account_manager_1.accountManager
        .collaboratorCan(uid, appName, logger)
        .then((col) => {
        return deployments_manager_1.deploymentsManager
            .findDeloymentByName(deploymentName, col.appid, logger)
            .then((deploymentInfo) => {
            if (lodash_1.default.isEmpty(deploymentInfo)) {
                throw new app_error_1.AppError('does not find the deployment');
            }
            const label = lodash_1.default.get(body, 'packageInfo.label');
            if (label) {
                return package_manager_1.packageManager
                    .findPackageInfoByDeploymentIdAndLabel(deploymentInfo.id, label)
                    .then((data) => {
                    return [deploymentInfo, data];
                });
            }
            const deploymentVersionId = deploymentInfo.last_deployment_version_id;
            return package_manager_1.packageManager
                .findLatestPackageInfoByDeployVersion(deploymentVersionId, logger)
                .then((data) => {
                return [deploymentInfo, data];
            });
        })
            .then(([deploymentInfo, packageInfo]) => {
            if (!packageInfo) {
                throw new app_error_1.AppError('does not find the packageInfo');
            }
            return package_manager_1.packageManager
                .modifyReleasePackage(packageInfo.id, body.packageInfo)
                .then(() => {
                // clear cache if exists.
                if (config_1.config.common.updateCheckCache) {
                    delay(2500).then(() => {
                        client_manager_1.clientManager.clearUpdateCheckCache(deploymentInfo.deployment_key, '*', '*', '*', logger);
                    });
                }
            });
        });
    })
        .then(() => {
        logger.info('modify release package success', {
            uid,
            appName,
            deploymentName,
        });
        res.send('');
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('modify release package failed', {
                uid,
                appName,
                deploymentName,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.post('/:appName/deployments/:sourceDeploymentName/promote/:destDeploymentName', middleware_1.checkToken, 
// eslint-disable-next-line max-lines-per-function
(req, res, next) => {
    const { logger, params, body } = req;
    const appName = lodash_1.default.trim(params.appName);
    const sourceDeploymentName = lodash_1.default.trim(params.sourceDeploymentName);
    const destDeploymentName = lodash_1.default.trim(params.destDeploymentName);
    const uid = req.users.id;
    logger.info('try promote package', {
        uid,
        appName,
        sourceDeploymentName,
        destDeploymentName,
        body: JSON.stringify(body),
    });
    account_manager_1.accountManager
        .collaboratorCan(uid, appName, logger)
        .then((col) => {
        const appId = col.appid;
        return Promise.all([
            deployments_manager_1.deploymentsManager.findDeloymentByName(sourceDeploymentName, appId, logger),
            deployments_manager_1.deploymentsManager.findDeloymentByName(destDeploymentName, appId, logger),
        ])
            .then(([sourceDeploymentInfo, destDeploymentInfo]) => {
            if (!sourceDeploymentInfo) {
                throw new app_error_1.AppError(`${sourceDeploymentName}  does not exist.`);
            }
            if (!destDeploymentInfo) {
                throw new app_error_1.AppError(`${destDeploymentName}  does not exist.`);
            }
            return [sourceDeploymentInfo, destDeploymentInfo];
        })
            .then(([sourceDeploymentInfo, destDeploymentInfo]) => {
            const promoteParams = lodash_1.default.get(body, 'packageInfo', {});
            // TODO: define packageInfo interface in packageManager
            lodash_1.default.set(promoteParams, 'promoteUid', uid);
            return package_manager_1.packageManager
                .promotePackage(sourceDeploymentInfo, destDeploymentInfo, promoteParams, logger)
                .then((packages) => {
                return [packages, destDeploymentInfo];
            });
        })
            .then(([packages, destDeploymentInfo]) => {
            if (packages) {
                delay(1000).then(() => {
                    package_manager_1.packageManager
                        .createDiffPackagesByLastNums(destDeploymentInfo.appid, packages, config_1.config.common.diffNums, logger)
                        .catch((e) => {
                        logger.error(e);
                    });
                });
            }
            // clear cache if exists.
            if (config_1.config.common.updateCheckCache) {
                delay(2500).then(() => {
                    client_manager_1.clientManager.clearUpdateCheckCache(destDeploymentInfo.deployment_key, '*', '*', '*', logger);
                });
            }
            return packages;
        });
    })
        .then((packages) => {
        logger.info('promote package success', {
            uid,
            appName,
            sourceDeploymentName,
            destDeploymentName,
        });
        res.send({ package: packages });
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('promote package failed', {
                uid,
                appName,
                sourceDeploymentName,
                destDeploymentName,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
function rollbackCb(req, res, next) {
    const { logger, params } = req;
    const appName = lodash_1.default.trim(params.appName);
    const deploymentName = lodash_1.default.trim(params.deploymentName);
    const targetLabel = lodash_1.default.trim(params.label);
    const uid = req.users.id;
    logger.info('try to rollback', {
        uid,
        appName,
        deploymentName,
        targetLabel,
    });
    account_manager_1.accountManager
        .collaboratorCan(uid, appName, logger)
        .then((col) => {
        return deployments_manager_1.deploymentsManager.findDeloymentByName(deploymentName, col.appid, logger);
    })
        .then((dep) => {
        return package_manager_1.packageManager
            .rollbackPackage(dep.last_deployment_version_id, targetLabel, uid, logger)
            .then((packageInfo) => {
            if (packageInfo) {
                delay(1000).then(() => {
                    package_manager_1.packageManager
                        .createDiffPackagesByLastNums(dep.appid, packageInfo, 1, logger)
                        .catch((e) => {
                        logger.error(e);
                    });
                });
            }
            // clear cache if exists.
            if (config_1.config.common.updateCheckCache) {
                delay(2500).then(() => {
                    logger.info('try clear update check cache');
                    client_manager_1.clientManager.clearUpdateCheckCache(dep.deployment_key, '*', '*', '*', logger);
                });
            }
            return packageInfo;
        });
    })
        .then(() => {
        logger.info('rollback success', {
            uid,
            appName,
            deploymentName,
            targetLabel,
        });
        res.send('ok');
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('rollback failed', {
                uid,
                appName,
                deploymentName,
                targetLabel,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
}
exports.appsRouter.post('/:appName/deployments/:deploymentName/rollback', middleware_1.checkToken, rollbackCb);
exports.appsRouter.post('/:appName/deployments/:deploymentName/rollback/:label', middleware_1.checkToken, rollbackCb);
exports.appsRouter.get('/:appName/collaborators', middleware_1.checkToken, (req, res, next) => {
    const { logger, params } = req;
    const appName = lodash_1.default.trim(params.appName);
    const uid = req.users.id;
    logger.info('try list collaborators', {
        uid,
        appName,
    });
    account_manager_1.accountManager
        .collaboratorCan(uid, appName, logger)
        .then((col) => {
        return collaborators_manager_1.collaboratorsManager.listCollaborators(col.appid);
    })
        .then((data) => {
        const rs = lodash_1.default.reduce(data, (result, value, key) => {
            let isCurrentAccount = false;
            if (lodash_1.default.eq(key, req.users.email)) {
                isCurrentAccount = true;
            }
            result[key] = {
                ...value,
                isCurrentAccount,
            };
            return result;
        }, {});
        logger.info('list collaborators success', {
            uid,
            appName,
        });
        res.send({ collaborators: rs });
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('list collaborators failed', {
                uid,
                appName,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.post('/:appName/collaborators/:email', middleware_1.checkToken, (req, res, next) => {
    const { logger, params } = req;
    const appName = lodash_1.default.trim(params.appName);
    const email = lodash_1.default.trim(params.email);
    const uid = req.users.id;
    logger.info('try to add collaborator', {
        uid,
        appName,
        email,
    });
    if (!validator_1.default.isEmail(email)) {
        res.status(406).send('Invalid Email!');
        return;
    }
    account_manager_1.accountManager
        .ownerCan(uid, appName, logger)
        .then((col) => {
        return account_manager_1.accountManager.findUserByEmail(email).then((data) => {
            return collaborators_manager_1.collaboratorsManager.addCollaborator(col.appid, data.id);
        });
    })
        .then((data) => {
        logger.info('add collaborator success', {
            uid,
            appName,
            email,
        });
        res.send(data);
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('add collaborator failed', {
                uid,
                appName,
                email,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.delete('/:appName/collaborators/:email', middleware_1.checkToken, (req, res, next) => {
    const { logger, params } = req;
    const appName = lodash_1.default.trim(params.appName);
    const email = lodash_1.default.trim(params.email);
    const uid = req.users.id;
    logger.info('try remove app collaborator', {
        uid,
        appName,
        email,
    });
    if (!validator_1.default.isEmail(email)) {
        res.status(406).send('Invalid Email!');
        return;
    }
    account_manager_1.accountManager
        .ownerCan(uid, appName, logger)
        .then((col) => {
        return account_manager_1.accountManager.findUserByEmail(email).then((data) => {
            if (lodash_1.default.eq(data.id, uid)) {
                throw new app_error_1.AppError("can't delete yourself!");
            }
            else {
                return collaborators_manager_1.collaboratorsManager.deleteCollaborator(col.appid, data.id);
            }
        });
    })
        .then(() => {
        logger.info('remove app collaborator success', {
            uid,
            appName,
            email,
        });
        res.send('');
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('remove app collaborator failed', {
                uid,
                appName,
                email,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.delete('/:appName', middleware_1.checkToken, (req, res, next) => {
    const { logger, params } = req;
    const appName = lodash_1.default.trim(params.appName);
    const uid = req.users.id;
    logger.info('try remove app', {
        uid,
        appName,
    });
    account_manager_1.accountManager
        .ownerCan(uid, appName, logger)
        .then((col) => {
        return app_manager_1.appManager.deleteApp(col.appid);
    })
        .then((data) => {
        logger.info('remove app success', {
            uid,
            appName,
        });
        res.send(data);
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('remove app failed', {
                uid,
                appName,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.patch('/:appName', middleware_1.checkToken, (req, res, next) => {
    const { logger, params, body } = req;
    const appName = lodash_1.default.trim(params.appName);
    const newAppName = lodash_1.default.trim(body.name);
    const uid = req.users.id;
    logger.info('try rename app', {
        uid,
        appName,
        newAppName,
    });
    if (lodash_1.default.isEmpty(newAppName)) {
        res.status(406).send('Please input name!');
        return;
    }
    account_manager_1.accountManager
        .ownerCan(uid, appName, logger)
        .then((col) => {
        return app_manager_1.appManager.findAppByName(uid, newAppName).then((appInfo) => {
            if (!lodash_1.default.isEmpty(appInfo)) {
                throw new app_error_1.AppError(`${newAppName} Exist!`);
            }
            return app_manager_1.appManager.modifyApp(col.appid, { name: newAppName });
        });
    })
        .then(() => {
        logger.info('rename app success', {
            uid,
            appName,
            newAppName,
        });
        res.send('');
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('rename app failed', {
                uid,
                appName,
                newAppName,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.post('/:appName/transfer/:email', middleware_1.checkToken, (req, res, next) => {
    const { logger, params } = req;
    const appName = lodash_1.default.trim(params.appName);
    const email = lodash_1.default.trim(params.email);
    const uid = req.users.id;
    logger.info('try transfer app', {
        uid,
        appName,
        to: email,
    });
    if (!validator_1.default.isEmail(email)) {
        res.status(406).send('Invalid Email!');
        return;
    }
    account_manager_1.accountManager
        .ownerCan(uid, appName, logger)
        .then((col) => {
        return account_manager_1.accountManager.findUserByEmail(email).then((data) => {
            if (lodash_1.default.eq(data.id, uid)) {
                throw new app_error_1.AppError("You can't transfer to yourself!");
            }
            return app_manager_1.appManager.transferApp(col.appid, uid, data.id);
        });
    })
        .then((data) => {
        logger.info('transfer app success', {
            uid,
            appName,
            to: email,
        });
        res.send(data);
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('transfer app failed', {
                uid,
                appName,
                to: email,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
exports.appsRouter.post('/', middleware_1.checkToken, 
// eslint-disable-next-line max-lines-per-function
(req, res, next) => {
    const { logger, body } = req;
    const uid = req.users.id;
    logger.info('try add app', {
        uid,
        body: JSON.stringify(body),
    });
    const appName = body.name;
    if (lodash_1.default.isEmpty(appName)) {
        res.status(406).send('Please input name!');
        return;
    }
    const osName = lodash_1.default.toLower(body.os);
    let os;
    if (osName === lodash_1.default.toLower(const_1.IOS_NAME)) {
        os = const_1.IOS;
    }
    else if (osName === lodash_1.default.toLower(const_1.ANDROID_NAME)) {
        os = const_1.ANDROID;
    }
    else if (osName === lodash_1.default.toLower(const_1.WINDOWS_NAME)) {
        os = const_1.WINDOWS;
    }
    else {
        res.status(406).send('Please input os [iOS|Android|Windows]!');
        return;
    }
    const platformName = lodash_1.default.toLower(body.platform);
    let platform;
    if (platformName === lodash_1.default.toLower(const_1.REACT_NATIVE_NAME)) {
        platform = const_1.REACT_NATIVE;
    }
    else if (platformName === lodash_1.default.toLower(const_1.CORDOVA_NAME)) {
        platform = const_1.CORDOVA;
    }
    else {
        res.status(406).send('Please input platform [React-Native|Cordova]!');
        return;
    }
    logger.info('try add app params check pass', {
        uid,
        appName,
    });
    app_manager_1.appManager
        .findAppByName(uid, appName)
        .then((appInfo) => {
        if (!lodash_1.default.isEmpty(appInfo)) {
            throw new app_error_1.AppError(`${appName} Exist!`);
        }
        return app_manager_1.appManager.addApp(uid, appName, os, platform, req.users.identical);
    })
        .then(() => {
        logger.info('add app success', {
            uid,
            name: appName,
        });
        const data = {
            name: appName,
            collaborators: { [req.users.email]: { permission: 'Owner' } },
        };
        res.send({ app: data });
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            logger.info('add app failed', {
                uid,
                name: appName,
                error: e.message,
            });
            res.status(406).send(e.message);
        }
        else {
            next(e);
        }
    });
});
